﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Windows.Forms;
using System.IO;
using System.Linq;
using System.Net;
using System.Xml.Linq;
using ThingM.Blink1;
using ThingM.Blink1.ColorProcessor;
using System.Runtime.InteropServices;




namespace blink_1_SysBarCTRL
{

    public partial class Form1 : Form
    {
        ThingM.Blink1.Blink1 blink1;
        [DllImport("user32.dll")]
        static extern bool GetLastInputInfo(ref LASTINPUTINFO plii);
       
        public string currentState = "avail";
        public Form1()
        {
            InitializeComponent();
        }

        internal struct LASTINPUTINFO
        {
            public uint cbSize;

            public uint dwTime;
        }
        private void Form1_Load(object sender, EventArgs e)
        {

            // setup timer to check user idle state
            this.CheckIdleTimer.Start();

            // setup defaults for application GUI
            availableToolStripMenuItem.Checked = true;
            outToLunchToolStripMenuItem.Checked = false;
            doNotDisturbToolStripMenuItem.Checked = false;

            try
            {
                blink1 = new Blink1();
                blink1.Open();

                int versionNumber = blink1.GetVersion();

                if (!blink1.IsConnected)
                {
                    MessageBox.Show("Could not open blink(1) device. Please ensure the device is plugged in and restart this application.", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    Application.Exit();
                }

                // stop any presets and fade to green
                blink1.StopPlayingPreset();

                blink1.FadeToColor(3000, new Rgb(0, 255, 0), false);
            }
            catch(Exception exp)
            {
                MessageBox.Show(exp.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private static uint GetIdleTime()
        {
            LASTINPUTINFO lastInPut = new LASTINPUTINFO();
            lastInPut.cbSize = (uint)System.Runtime.InteropServices.Marshal.SizeOf(lastInPut);
            GetLastInputInfo(ref lastInPut);

            return ((uint)Environment.TickCount - lastInPut.dwTime);
        }
        private void availableToolStripMenuItem_Click(object sender, EventArgs e)
        {
            availableToolStripMenuItem.Checked = true;
            outToLunchToolStripMenuItem.Checked = false;
            doNotDisturbToolStripMenuItem.Checked = false;
            currentState = "avail";

            if (blink1.IsConnected)
            {
                blink1.FadeToColor(3000, new Rgb(0, 50, 0), false); //green
            }
            else
            {
                MessageBox.Show("Blink(1) not connected. Please reconnect the blink1 and restart this application.", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void outToLunchToolStripMenuItem_Click(object sender, EventArgs e)
        {
            availableToolStripMenuItem.Checked = false;
            outToLunchToolStripMenuItem.Checked = true;
            doNotDisturbToolStripMenuItem.Checked = false;
            currentState = "out";

            if (blink1.IsConnected)
            {
                blink1.FadeToColor(3000, new Rgb(50, 50, 0), false); //yellow
            }
            else {
                MessageBox.Show("Blink(1) not connected. Please reconnect the blink1 and restart this application.", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void doNotDisturbToolStripMenuItem_Click(object sender, EventArgs e)
        {
            availableToolStripMenuItem.Checked = false;
            outToLunchToolStripMenuItem.Checked = false;
            doNotDisturbToolStripMenuItem.Checked = true;
            currentState = "DND";

            if (blink1.IsConnected)
            {
                blink1.FadeToColor(3000, new Rgb(50, 0, 0), false); //red
            }
            else
            {
                MessageBox.Show("Blink(1) not connected. Please reconnect the blink1 and restart this application.", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void exitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            blink1.SetColor(new Rgb(0, 0, 0));
            Application.Exit();
        }

        private void SetBallonTip(string title, string text)
        {
            notifyIcon1.BalloonTipTitle = title;
            notifyIcon1.BalloonTipText = text;
            notifyIcon1.BalloonTipIcon = ToolTipIcon.Info;
            notifyIcon1.ShowBalloonTip(10000);
        }

        private void CheckIdleTimer_Tick(object sender, EventArgs e)
        {
            // if user id idle more than 5 minutes, change the blink(1) to "out to lunch"
            if (GetIdleTime() > 300000)
            {
                outToLunchToolStripMenuItem_Click(null, null);
                currentState = "away";
                SetBallonTip("Out to lunch?", "You have been set as out to lunch because you were inactive for 5 minutes.");
            }
            else
            {
                if (currentState == "away")
                {
                    availableToolStripMenuItem_Click(null, null);
                }
            }
        }
    }
}
